#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_types.h"
#include "driverlib/debug.h"
#include "driverlib/gpio.h"
#include "driverlib/interrupt.h"
#include "driverlib/sysctl.h"
#include "driverlib/uart.h"
#include "drivers/rit128x96x4.h"
#include "utils/ustdlib.h"
#include <string.h>
#include "driverlib/timer.h"


#define KEY_ENTER_VALUE         15
#define KEY_DOWN_VALUE          29
#define KEY_UP_VALUE            30
#define KEY_RIGHT_VALUE         23
#define KEY_LEFT_VALUE          27


static char buf[96];
static char artist[21];
static char song[21];
static char album[21];
static char menuStr[16];

static int menuInt = 0;
int lenght = 0;

void initHw(void);
void printInfo(void);
void ButtonIntHandler(void);
void Timer0IntHandler(void);
void UARTSend(const unsigned char *pucBuffer, unsigned long ulCount);


void initHw(){
    //
    // Set the clocking to run directly from the crystal.
    //
    SysCtlClockSet(SYSCTL_SYSDIV_1 | SYSCTL_USE_OSC | SYSCTL_OSC_MAIN | SYSCTL_XTAL_8MHZ);

    //
    // Initialize the OLED display and write status.
    //
    RIT128x96x4Init(3000000);

    //
    // Enable the peripherals used by this example.
    //
    SysCtlPeripheralEnable(SYSCTL_PERIPH_UART0);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOA);
    //butons
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOE);
    SysCtlPeripheralEnable(SYSCTL_PERIPH_GPIOF);
    //timer
    SysCtlPeripheralEnable(SYSCTL_PERIPH_TIMER0);

    //
    // Enable processor interrupts.
    //
    IntMasterEnable();

    //
    // Set GPIO A0 and A1 as UART pins.
    //
    GPIOPinTypeUART(GPIO_PORTA_BASE, GPIO_PIN_0 | GPIO_PIN_1);

    //
    // Configure the UART for 115,200, 8-N-1 operation.
    //
    UARTConfigSetExpClk(UART0_BASE, SysCtlClockGet(), 115200,
            (UART_CONFIG_WLEN_8 | UART_CONFIG_STOP_ONE |
            UART_CONFIG_PAR_NONE));

    //
    // Configure the GPIOs used to read the state of the on-board push buttons.
    //
    GPIOPinTypeGPIOInput(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3);
    GPIOPadConfigSet(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);

    GPIOPinTypeGPIOInput(GPIO_PORTF_BASE, GPIO_PIN_1);
    GPIOPadConfigSet(GPIO_PORTF_BASE, GPIO_PIN_1, GPIO_STRENGTH_2MA, GPIO_PIN_TYPE_STD_WPU);

    //
    // Enable the UART interrupt.
    //
    IntEnable(INT_UART0);
    UARTIntEnable(UART0_BASE, UART_INT_RX | UART_INT_RT);


    // Configure GPIO Block E Pin 2 and 3 to detect a falling edge
    //   (i.e. when the LEFT or RIGHT button is pressed).
    GPIOIntTypeSet(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3, GPIO_FALLING_EDGE);
    GPIOIntTypeSet(GPIO_PORTF_BASE, GPIO_PIN_1, GPIO_FALLING_EDGE);

    // Enable GPIO Port E, Pin 0 - 3 to generate an interrupt
    GPIOPinIntEnable(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3);
    GPIOPinIntEnable(GPIO_PORTF_BASE, GPIO_PIN_1);


    // Enable the Interrupt for GPIO block E and F in the NVIC
    IntEnable(INT_GPIOE);
    IntEnable(INT_GPIOF);

    //
    // Configure the two 32-bit periodic timers.
    //
    TimerConfigure(TIMER0_BASE, TIMER_CFG_32_BIT_PER);
    TimerLoadSet(TIMER0_BASE, TIMER_A, 5 * SysCtlClockGet());

    //
    // Setup the interrupts for the timer timeouts.
    //
    IntEnable(INT_TIMER0A);
    TimerIntEnable(TIMER0_BASE, TIMER_TIMA_TIMEOUT);

    //
    // Enable the timers.
    //
    TimerEnable(TIMER0_BASE, TIMER_A);

}

void printInfo(void) {

    RIT128x96x4Clear();
    RIT128x96x4StringDraw(song, 2, 10, 13);
    RIT128x96x4StringDraw(artist, 2, 25, 10);
    RIT128x96x4StringDraw(album, 2, 40, 7);
    RIT128x96x4StringDraw(menuStr, 2, 80, 10);

}


void UARTIntHandler(void) {
    unsigned long ulStatus;

    char c;

    ulStatus = UARTIntStatus(UART0_BASE, true);

    UARTIntClear(UART0_BASE, ulStatus);


    while (UARTCharsAvail(UART0_BASE)) {

        c = UARTCharGetNonBlocking(UART0_BASE);
        buf[lenght] = c;
        lenght++;

        if (c == 0xF3) {
            buf[lenght - 1] = '\0';
            strcpy(song, buf);
            lenght = 0;
        }
        if (c == 0xF6) {
            buf[lenght - 1] = '\0';
            strcpy(artist, buf);
            lenght = 0;
        }
        if (c == 0xF9) {
            buf[lenght - 1] = '\0';
            strcpy(album, buf);
            lenght = 0;
            printInfo();
        }
    }
}

void UARTSend(const unsigned char *pucBuffer, unsigned long ulCount) {

    while (ulCount--) {
        UARTCharPutNonBlocking(UART0_BASE, *pucBuffer++);
    }
}

int main(void) {

    initHw();

    strcpy(artist, "artist");
    strcpy(song, "song");
    strcpy(album, "album");

    printInfo();
    RIT128x96x4StringDraw("Next song", 2, 80, 10);

    while (1) {

    }
}

void ButtonIntHandler(void) {
    // Variable for pin status (LEFT or RIGHT)
    long pinStatus;

    // Delay for debouncing
    volatile unsigned long delay;

    char message[16];

    // Wait a bit
    for (delay = 0; delay < 100000; delay++) {
    }

    // Read the state again
    pinStatus = (GPIOPinRead(GPIO_PORTE_BASE, (GPIO_PIN_0 | GPIO_PIN_1 |
            GPIO_PIN_2 | GPIO_PIN_3)) |
            (GPIOPinRead(GPIO_PORTF_BASE, GPIO_PIN_1) << 3));

    // Clear the GPIO interrupt.
    GPIOPinIntClear(GPIO_PORTE_BASE, GPIO_PIN_0 | GPIO_PIN_1 | GPIO_PIN_2 | GPIO_PIN_3);
    GPIOPinIntClear(GPIO_PORTF_BASE, GPIO_PIN_1);


    switch (pinStatus) {
        case KEY_DOWN_VALUE:
            strcpy(message, "VOLUME_DOWN");
            UARTSend(message, strlen(message));
            break;

        case KEY_ENTER_VALUE:
            //tady se rozhodne co se ode�le
            switch (menuInt) {
                case 0:
                    strcpy(message, "NEXT_SONG");
                    UARTSend(message, strlen(message));
                    break;
                case 1:
                    strcpy(message, "PREVIOUS_SONG");
                    UARTSend(message, strlen(message));
                    break;
                case 2:
                    strcpy(message, "PAUSE");
                    UARTSend(message, strlen(message));
                    break;
                case 3:
                    strcpy(message, "PLAY");
                    UARTSend(message, strlen(message));
                    break;
                default:
                    break;
            }
            break;
        case KEY_LEFT_VALUE:
        case KEY_RIGHT_VALUE:

            //pohyb v menu
            if (pinStatus == KEY_LEFT_VALUE) {
                menuInt = menuInt - 1;
                if (menuInt < 0) menuInt = 3;
            } else {
                menuInt = menuInt + 1;
                //MAX polo�ek v menu
                if (menuInt > 3) menuInt = 0;
            }

            //vypis polozky
            switch (menuInt) {
                case 0:
                    strcpy(menuStr, "Next song");
                    break;
                case 1:
                    strcpy(menuStr, "Previous song");
                    break;
                case 2:
                    strcpy(menuStr, "Pause");
                    break;
                case 3:
                    strcpy(menuStr, "Play");
                    break;
                default:
                    menuInt = 0;
                    strcpy(menuStr, "Next song");
                    break;
            }
            printInfo();


            break;
        case KEY_UP_VALUE:
            strcpy(message, "VOLUME_UP");
            UARTSend(message, strlen(message));
            break;
        default:
            //strcpy(message, "ERROR");
            break;
    }
}

void Timer0IntHandler(void) {

    TimerIntClear(TIMER0_BASE, TIMER_TIMA_TIMEOUT);
    char message[5];
    strcpy(message, "INFO");
    UARTSend(message, strlen(message));

}
